module Lib
    ( Geometry ( Rectangle, Triangle, Semicircle )
    , Location ( Location )
    , Mirror ( N, X, Y, XY )
    , Swap ( NS, S )
    , Negation ( Positive, Negative )
    , Figure ( Figure )
    , Centroid ( area, xb, yb )
    , Compound ( Compound )
    , centroid
    ) where

data Geometry = Rectangle Double Double -- width x height
              | Triangle Double Double  -- base x height
              | Semicircle Double       -- radius
              deriving Show

data Location = Location Double Double deriving Show -- x, y
data Mirror = N | X | Y | XY deriving Show
data Swap = NS | S deriving Show
data Negation = Positive | Negative deriving Show
data Figure = Figure Geometry Location Mirror Swap Negation deriving Show

newtype Compound = Compound [Figure] deriving Show

class Centroid a where
  area :: a -> Double
  xb :: a -> Double
  yb :: a -> Double

instance Centroid Geometry where
  area (Rectangle w h) = w * h
  area (Triangle b h) = b * h / 2
  area (Semicircle r) = pi * r ^ 2 / 2

  xb (Rectangle w _) = w / 2
  xb (Triangle b _) = b / 3
  xb (Semicircle r) = 0

  yb (Rectangle _ h) = h / 2
  yb (Triangle _ h) = h / 3
  yb (Semicircle r) = 4 * r / (3 * pi)

-- xb' / yb' swap the actual xbs and ybs
-- mirror reflects across the axis, origin stays (0, 0)
-- negate' negates an area depending on whether its positive or negative space

instance Centroid Figure where
  area (Figure geometry _ _ _ negation) = negate' negation $ area geometry
    where negate' Positive = id
          negate' Negative = negate

  xb (Figure geometry (Location dx _) axis swapped _) =
    (+ dx) . mirror axis . xb' swapped $ geometry

    where mirror Y = negate
          mirror XY = negate
          mirror _ = id

          xb' NS = xb
          xb' S = yb

  yb (Figure geometry (Location _ dy) axis swapped _) =
    (+ dy) . mirror axis . yb' swapped $ geometry

    where mirror X = negate
          mirror XY = negate
          mirror _ = id

          yb' NS = yb
          yb' S = xb

dist f (Compound figures) = (sum $ map (\figure -> area figure * f figure) figures)
                            / (sum $ map area figures)

instance Centroid Compound where
  area (Compound figures) = sum $ map area figures
  xb = dist xb
  yb = dist yb

centroid :: Centroid a => a -> Location
centroid figure = Location (xb figure) (yb figure)
